# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .variable import Variable


class ModelMetaResponse(UniversalBaseModel):
    variables: typing.List[Variable]
    type: str
    expected_runs: typing.Optional[typing.List[int]] = None
    expected_hours: typing.Optional[typing.Dict[str, typing.Optional[int]]] = None
    latest_run: typing.Optional[str] = None
    min_date: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CreateLayerApiV1DashboardLayersPostResponse,
    GetLayerApiV1DashboardLayersLayerIdGetResponse,
    ListLayersApiV1DashboardLayersGetResponseItem,
    UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType,
    UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType,
    UpdateLayerApiV1DashboardLayersLayerIdPutResponse,
)

__all__ = [
    "CreateLayerApiV1DashboardLayersPostResponse",
    "GetLayerApiV1DashboardLayersLayerIdGetResponse",
    "ListLayersApiV1DashboardLayersGetResponseItem",
    "UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType",
    "UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType",
    "UpdateLayerApiV1DashboardLayersLayerIdPutResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.dashboard import Dashboard
from ..types.react_grid_layout_config import ReactGridLayoutConfig
from ..types.unified_dashboard_response import UnifiedDashboardResponse
from .raw_client import AsyncRawDashboardsClient, RawDashboardsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DashboardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDashboardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDashboardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDashboardsClient
        """
        return self._raw_client

    def list_dashboards(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Dashboard]:
        """
        List all dashboards for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Dashboard]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.list_dashboards()
        """
        _response = self._raw_client.list_dashboards(request_options=request_options)
        return _response.data

    def create_dashboard(
        self,
        *,
        name: str,
        components: typing.Optional[typing.Sequence[str]] = OMIT,
        react_grid_layout_config: typing.Optional[ReactGridLayoutConfig] = OMIT,
        component_state: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Dashboard:
        """
        Create a new dashboard.

        Parameters
        ----------
        name : str

        components : typing.Optional[typing.Sequence[str]]

        react_grid_layout_config : typing.Optional[ReactGridLayoutConfig]

        component_state : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Dashboard
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.create_dashboard(
            name="name",
        )
        """
        _response = self._raw_client.create_dashboard(
            name=name,
            components=components,
            react_grid_layout_config=react_grid_layout_config,
            component_state=component_state,
            request_options=request_options,
        )
        return _response.data

    def get_dashboard(self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Dashboard:
        """
        Retrieve a single dashboard.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Dashboard
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.get_dashboard(
            dashboard_id="dashboard_id",
        )
        """
        _response = self._raw_client.get_dashboard(dashboard_id, request_options=request_options)
        return _response.data

    def update_dashboard_layout(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Update a dashboard's layout and component state.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.update_dashboard_layout(
            dashboard_id="dashboard_id",
        )
        """
        _response = self._raw_client.update_dashboard_layout(dashboard_id, request_options=request_options)
        return _response.data

    def delete_dashboard(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete a dashboard.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.delete_dashboard(
            dashboard_id="dashboard_id",
        )
        """
        _response = self._raw_client.delete_dashboard(dashboard_id, request_options=request_options)
        return _response.data

    def get_dashboard_full(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UnifiedDashboardResponse:
        """
        Retrieve a dashboard with all its components, layers, and datasets in a single request.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UnifiedDashboardResponse
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.dashboards.get_dashboard_full(
            dashboard_id="dashboard_id",
        )
        """
        _response = self._raw_client.get_dashboard_full(dashboard_id, request_options=request_options)
        return _response.data


class AsyncDashboardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDashboardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDashboardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDashboardsClient
        """
        return self._raw_client

    async def list_dashboards(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Dashboard]:
        """
        List all dashboards for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Dashboard]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.list_dashboards()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_dashboards(request_options=request_options)
        return _response.data

    async def create_dashboard(
        self,
        *,
        name: str,
        components: typing.Optional[typing.Sequence[str]] = OMIT,
        react_grid_layout_config: typing.Optional[ReactGridLayoutConfig] = OMIT,
        component_state: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Dashboard:
        """
        Create a new dashboard.

        Parameters
        ----------
        name : str

        components : typing.Optional[typing.Sequence[str]]

        react_grid_layout_config : typing.Optional[ReactGridLayoutConfig]

        component_state : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Dashboard
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.create_dashboard(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_dashboard(
            name=name,
            components=components,
            react_grid_layout_config=react_grid_layout_config,
            component_state=component_state,
            request_options=request_options,
        )
        return _response.data

    async def get_dashboard(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Dashboard:
        """
        Retrieve a single dashboard.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Dashboard
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.get_dashboard(
                dashboard_id="dashboard_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_dashboard(dashboard_id, request_options=request_options)
        return _response.data

    async def update_dashboard_layout(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Update a dashboard's layout and component state.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.update_dashboard_layout(
                dashboard_id="dashboard_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_dashboard_layout(dashboard_id, request_options=request_options)
        return _response.data

    async def delete_dashboard(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete a dashboard.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.delete_dashboard(
                dashboard_id="dashboard_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_dashboard(dashboard_id, request_options=request_options)
        return _response.data

    async def get_dashboard_full(
        self, dashboard_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UnifiedDashboardResponse:
        """
        Retrieve a dashboard with all its components, layers, and datasets in a single request.

        Parameters
        ----------
        dashboard_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UnifiedDashboardResponse
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.dashboards.get_dashboard_full(
                dashboard_id="dashboard_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_dashboard_full(dashboard_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .dashboard import Dashboard
from .dataset import Dataset
from .unified_dashboard_response_components_item import UnifiedDashboardResponseComponentsItem
from .unified_dashboard_response_layers_item import UnifiedDashboardResponseLayersItem


class UnifiedDashboardResponse(UniversalBaseModel):
    """
    Unified response containing all dashboard data in a single request.
    """

    dashboard: Dashboard
    components: typing.List[UnifiedDashboardResponseComponentsItem]
    layers: typing.List[UnifiedDashboardResponseLayersItem]
    datasets: typing.List[Dataset]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class MapConfig(UniversalBaseModel):
    """
    Configuration specific to a map component.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Initial map center latitude.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Initial map center longitude.
    """

    zoom: typing.Optional[int] = pydantic.Field(default=None)
    """
    Initial map zoom level
    """

    map_style: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="mapStyle")] = pydantic.Field(
        default=None
    )
    """
    Mapbox style URL
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.ui_config import UiConfig
from .types.create_layer_api_v_1_dashboard_layers_post_response import CreateLayerApiV1DashboardLayersPostResponse
from .types.get_layer_api_v_1_dashboard_layers_layer_id_get_response import (
    GetLayerApiV1DashboardLayersLayerIdGetResponse,
)
from .types.list_layers_api_v_1_dashboard_layers_get_response_item import ListLayersApiV1DashboardLayersGetResponseItem
from .types.update_layer_api_v_1_dashboard_layers_layer_id_put_request_map_layer_type import (
    UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType,
)
from .types.update_layer_api_v_1_dashboard_layers_layer_id_put_request_weather_layer_type import (
    UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType,
)
from .types.update_layer_api_v_1_dashboard_layers_layer_id_put_response import (
    UpdateLayerApiV1DashboardLayersLayerIdPutResponse,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLayersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_layers(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[ListLayersApiV1DashboardLayersGetResponseItem]]:
        """
        List all layers for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[ListLayersApiV1DashboardLayersGetResponseItem]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/layers",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ListLayersApiV1DashboardLayersGetResponseItem],
                    parse_obj_as(
                        type_=typing.List[ListLayersApiV1DashboardLayersGetResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_layer(
        self,
        *,
        name: str,
        type: str,
        weather_model_id: typing.Optional[str] = None,
        variable: typing.Optional[str] = None,
        altitude: typing.Optional[int] = None,
        run_hour: typing.Optional[int] = None,
        run_day: typing.Optional[str] = None,
        dataset_id: typing.Optional[str] = None,
        deckgl_props: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        ui_config: typing.Optional[UiConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateLayerApiV1DashboardLayersPostResponse]:
        """
        Create a new layer using the discriminator pattern.

        The 'type' field determines which layer type to create:
        - Weather layers: 'wl:RasterLayer', 'wl:ParticleLayer', 'wl:IsolineLayer'
        - Map layers: 'ScatterPlotLayer', 'GeoJsonLayer'

        Parameters
        ----------
        name : str

        type : str

        weather_model_id : typing.Optional[str]

        variable : typing.Optional[str]

        altitude : typing.Optional[int]

        run_hour : typing.Optional[int]

        run_day : typing.Optional[str]

        dataset_id : typing.Optional[str]

        deckgl_props : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        ui_config : typing.Optional[UiConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateLayerApiV1DashboardLayersPostResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/layers",
            method="POST",
            params={
                "name": name,
                "type": type,
                "weather_model_id": weather_model_id,
                "variable": variable,
                "altitude": altitude,
                "run_hour": run_hour,
                "run_day": run_day,
                "dataset_id": dataset_id,
            },
            json={
                "deckgl_props": deckgl_props,
                "ui_config": convert_and_respect_annotation_metadata(
                    object_=ui_config, annotation=UiConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLayerApiV1DashboardLayersPostResponse,
                    parse_obj_as(
                        type_=CreateLayerApiV1DashboardLayersPostResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_layer(
        self, layer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetLayerApiV1DashboardLayersLayerIdGetResponse]:
        """
        Get a specific layer by ID.

        Parameters
        ----------
        layer_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLayerApiV1DashboardLayersLayerIdGetResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLayerApiV1DashboardLayersLayerIdGetResponse,
                    parse_obj_as(
                        type_=GetLayerApiV1DashboardLayersLayerIdGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_layer(
        self,
        layer_id: str,
        *,
        name: str,
        weather_model_id: typing.Optional[str] = None,
        variable: typing.Optional[str] = None,
        altitude: typing.Optional[int] = None,
        run_hour: typing.Optional[int] = None,
        run_day: typing.Optional[str] = None,
        weather_layer_type: typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType] = None,
        map_layer_type: typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType] = None,
        dataset_id: typing.Optional[str] = None,
        deckgl_props: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        ui_config: typing.Optional[UiConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateLayerApiV1DashboardLayersLayerIdPutResponse]:
        """
        Update an existing layer.

        Parameters
        ----------
        layer_id : str

        name : str

        weather_model_id : typing.Optional[str]

        variable : typing.Optional[str]

        altitude : typing.Optional[int]

        run_hour : typing.Optional[int]

        run_day : typing.Optional[str]

        weather_layer_type : typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType]

        map_layer_type : typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType]

        dataset_id : typing.Optional[str]

        deckgl_props : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        ui_config : typing.Optional[UiConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateLayerApiV1DashboardLayersLayerIdPutResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="PUT",
            params={
                "name": name,
                "weather_model_id": weather_model_id,
                "variable": variable,
                "altitude": altitude,
                "run_hour": run_hour,
                "run_day": run_day,
                "weather_layer_type": weather_layer_type,
                "map_layer_type": map_layer_type,
                "dataset_id": dataset_id,
            },
            json={
                "deckgl_props": deckgl_props,
                "ui_config": convert_and_respect_annotation_metadata(
                    object_=ui_config, annotation=UiConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLayerApiV1DashboardLayersLayerIdPutResponse,
                    parse_obj_as(
                        type_=UpdateLayerApiV1DashboardLayersLayerIdPutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_layer(
        self, layer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Delete a layer by ID.

        Parameters
        ----------
        layer_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLayersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_layers(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[ListLayersApiV1DashboardLayersGetResponseItem]]:
        """
        List all layers for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[ListLayersApiV1DashboardLayersGetResponseItem]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/layers",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ListLayersApiV1DashboardLayersGetResponseItem],
                    parse_obj_as(
                        type_=typing.List[ListLayersApiV1DashboardLayersGetResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_layer(
        self,
        *,
        name: str,
        type: str,
        weather_model_id: typing.Optional[str] = None,
        variable: typing.Optional[str] = None,
        altitude: typing.Optional[int] = None,
        run_hour: typing.Optional[int] = None,
        run_day: typing.Optional[str] = None,
        dataset_id: typing.Optional[str] = None,
        deckgl_props: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        ui_config: typing.Optional[UiConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateLayerApiV1DashboardLayersPostResponse]:
        """
        Create a new layer using the discriminator pattern.

        The 'type' field determines which layer type to create:
        - Weather layers: 'wl:RasterLayer', 'wl:ParticleLayer', 'wl:IsolineLayer'
        - Map layers: 'ScatterPlotLayer', 'GeoJsonLayer'

        Parameters
        ----------
        name : str

        type : str

        weather_model_id : typing.Optional[str]

        variable : typing.Optional[str]

        altitude : typing.Optional[int]

        run_hour : typing.Optional[int]

        run_day : typing.Optional[str]

        dataset_id : typing.Optional[str]

        deckgl_props : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        ui_config : typing.Optional[UiConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateLayerApiV1DashboardLayersPostResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/layers",
            method="POST",
            params={
                "name": name,
                "type": type,
                "weather_model_id": weather_model_id,
                "variable": variable,
                "altitude": altitude,
                "run_hour": run_hour,
                "run_day": run_day,
                "dataset_id": dataset_id,
            },
            json={
                "deckgl_props": deckgl_props,
                "ui_config": convert_and_respect_annotation_metadata(
                    object_=ui_config, annotation=UiConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLayerApiV1DashboardLayersPostResponse,
                    parse_obj_as(
                        type_=CreateLayerApiV1DashboardLayersPostResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_layer(
        self, layer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetLayerApiV1DashboardLayersLayerIdGetResponse]:
        """
        Get a specific layer by ID.

        Parameters
        ----------
        layer_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLayerApiV1DashboardLayersLayerIdGetResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLayerApiV1DashboardLayersLayerIdGetResponse,
                    parse_obj_as(
                        type_=GetLayerApiV1DashboardLayersLayerIdGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_layer(
        self,
        layer_id: str,
        *,
        name: str,
        weather_model_id: typing.Optional[str] = None,
        variable: typing.Optional[str] = None,
        altitude: typing.Optional[int] = None,
        run_hour: typing.Optional[int] = None,
        run_day: typing.Optional[str] = None,
        weather_layer_type: typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType] = None,
        map_layer_type: typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType] = None,
        dataset_id: typing.Optional[str] = None,
        deckgl_props: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        ui_config: typing.Optional[UiConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateLayerApiV1DashboardLayersLayerIdPutResponse]:
        """
        Update an existing layer.

        Parameters
        ----------
        layer_id : str

        name : str

        weather_model_id : typing.Optional[str]

        variable : typing.Optional[str]

        altitude : typing.Optional[int]

        run_hour : typing.Optional[int]

        run_day : typing.Optional[str]

        weather_layer_type : typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestWeatherLayerType]

        map_layer_type : typing.Optional[UpdateLayerApiV1DashboardLayersLayerIdPutRequestMapLayerType]

        dataset_id : typing.Optional[str]

        deckgl_props : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        ui_config : typing.Optional[UiConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateLayerApiV1DashboardLayersLayerIdPutResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="PUT",
            params={
                "name": name,
                "weather_model_id": weather_model_id,
                "variable": variable,
                "altitude": altitude,
                "run_hour": run_hour,
                "run_day": run_day,
                "weather_layer_type": weather_layer_type,
                "map_layer_type": map_layer_type,
                "dataset_id": dataset_id,
            },
            json={
                "deckgl_props": deckgl_props,
                "ui_config": convert_and_respect_annotation_metadata(
                    object_=ui_config, annotation=UiConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLayerApiV1DashboardLayersLayerIdPutResponse,
                    parse_obj_as(
                        type_=UpdateLayerApiV1DashboardLayersLayerIdPutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_layer(
        self, layer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Delete a layer by ID.

        Parameters
        ----------
        layer_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/layers/{jsonable_encoder(layer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .dashboard import Dashboard
from .dataformat import Dataformat
from .dataset import Dataset
from .geometrytype import Geometrytype
from .geospatial_data_type import GeospatialDataType
from .http_validation_error import HttpValidationError
from .image_meta import ImageMeta
from .image_meta_response import ImageMetaResponse
from .layout import Layout
from .live_data_source import LiveDataSource
from .map_component import MapComponent
from .map_config import MapConfig
from .map_layer import MapLayer
from .map_layer_type import MapLayerType
from .model_meta import ModelMeta
from .model_meta_response import ModelMetaResponse
from .model_runs_response import ModelRunsResponse
from .react_grid_layout_config import ReactGridLayoutConfig
from .source import Source, Source_Live, Source_Static
from .static_data_source import StaticDataSource
from .tabular_data_type import TabularDataType
from .time_series_component import TimeSeriesComponent
from .time_series_config import TimeSeriesConfig
from .time_series_data_point import TimeSeriesDataPoint
from .time_series_data_type import TimeSeriesDataType
from .typeconfig import Typeconfig, Typeconfig_Geospatial, Typeconfig_Tabular, Typeconfig_Timeseries
from .ui_config import UiConfig
from .ui_control_config import UiControlConfig
from .ui_control_config_data_type import UiControlConfigDataType
from .ui_control_type import UiControlType
from .unified_dashboard_response import UnifiedDashboardResponse
from .unified_dashboard_response_components_item import (
    UnifiedDashboardResponseComponentsItem,
    UnifiedDashboardResponseComponentsItem_Map,
    UnifiedDashboardResponseComponentsItem_Timeseries,
    UnifiedDashboardResponseComponentsItem_WeatherMap,
)
from .unified_dashboard_response_layers_item import UnifiedDashboardResponseLayersItem
from .unit import Unit
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .variable import Variable
from .weather_layer import WeatherLayer
from .weather_layer_type import WeatherLayerType
from .weather_map_component import WeatherMapComponent
from .weather_map_config import WeatherMapConfig

__all__ = [
    "Dashboard",
    "Dataformat",
    "Dataset",
    "Geometrytype",
    "GeospatialDataType",
    "HttpValidationError",
    "ImageMeta",
    "ImageMetaResponse",
    "Layout",
    "LiveDataSource",
    "MapComponent",
    "MapConfig",
    "MapLayer",
    "MapLayerType",
    "ModelMeta",
    "ModelMetaResponse",
    "ModelRunsResponse",
    "ReactGridLayoutConfig",
    "Source",
    "Source_Live",
    "Source_Static",
    "StaticDataSource",
    "TabularDataType",
    "TimeSeriesComponent",
    "TimeSeriesConfig",
    "TimeSeriesDataPoint",
    "TimeSeriesDataType",
    "Typeconfig",
    "Typeconfig_Geospatial",
    "Typeconfig_Tabular",
    "Typeconfig_Timeseries",
    "UiConfig",
    "UiControlConfig",
    "UiControlConfigDataType",
    "UiControlType",
    "UnifiedDashboardResponse",
    "UnifiedDashboardResponseComponentsItem",
    "UnifiedDashboardResponseComponentsItem_Map",
    "UnifiedDashboardResponseComponentsItem_Timeseries",
    "UnifiedDashboardResponseComponentsItem_WeatherMap",
    "UnifiedDashboardResponseLayersItem",
    "Unit",
    "ValidationError",
    "ValidationErrorLocItem",
    "Variable",
    "WeatherLayer",
    "WeatherLayerType",
    "WeatherMapComponent",
    "WeatherMapConfig",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.map_component import MapComponent
from ..types.map_config import MapConfig
from ..types.time_series_component import TimeSeriesComponent
from ..types.time_series_config import TimeSeriesConfig
from ..types.weather_map_component import WeatherMapComponent
from ..types.weather_map_config import WeatherMapConfig
from .raw_client import AsyncRawComponentsClient, RawComponentsClient
from .types.list_components_api_v_1_dashboard_components_get_response_item import (
    ListComponentsApiV1DashboardComponentsGetResponseItem,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ComponentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawComponentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawComponentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawComponentsClient
        """
        return self._raw_client

    def create_map(
        self,
        *,
        name: str,
        config: MapConfig,
        layer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapComponent:
        """
        Create a new map component.

        Parameters
        ----------
        name : str

        config : MapConfig

        layer_ids : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MapComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import MapConfig, RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.create_map(
            name="name",
            config=MapConfig(),
        )
        """
        _response = self._raw_client.create_map(
            name=name, config=config, layer_ids=layer_ids, request_options=request_options
        )
        return _response.data

    def create_timeseries(
        self,
        *,
        name: str,
        title: str,
        datasets: typing.Sequence[str],
        y_axis_label: typing.Optional[str] = OMIT,
        unit: typing.Optional[str] = OMIT,
        formula: typing.Optional[str] = OMIT,
        echarts_options: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesComponent:
        """
        Create a new time series component.

        Parameters
        ----------
        name : str

        title : str
            The user-defined title of the chart.

        datasets : typing.Sequence[str]
            List of timeseries dataset IDs to display on this chart.

        y_axis_label : typing.Optional[str]
            Label for the Y-axis.

        unit : typing.Optional[str]
            The unit for the Y-axis values, e.g., 'MW'.

        formula : typing.Optional[str]
            Optional formula to combine datasets, e.g., 'dataset_1 + dataset_2'

        echarts_options : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional ECharts options for the chart.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.create_timeseries(
            name="name",
            title="title",
            datasets=["datasets"],
        )
        """
        _response = self._raw_client.create_timeseries(
            name=name,
            title=title,
            datasets=datasets,
            y_axis_label=y_axis_label,
            unit=unit,
            formula=formula,
            echarts_options=echarts_options,
            request_options=request_options,
        )
        return _response.data

    def create_weather_map(
        self, *, name: str, config: WeatherMapConfig, request_options: typing.Optional[RequestOptions] = None
    ) -> WeatherMapComponent:
        """
        Create a new weather-map component.

        Parameters
        ----------
        name : str

        config : WeatherMapConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WeatherMapComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi, WeatherMapConfig

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.create_weather_map(
            name="name",
            config=WeatherMapConfig(),
        )
        """
        _response = self._raw_client.create_weather_map(name=name, config=config, request_options=request_options)
        return _response.data

    def list_components(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ListComponentsApiV1DashboardComponentsGetResponseItem]:
        """
        List all components in the user's library.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ListComponentsApiV1DashboardComponentsGetResponseItem]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.list_components()
        """
        _response = self._raw_client.list_components(request_options=request_options)
        return _response.data

    def get_component(
        self, component_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Retrieve a single component.

        Parameters
        ----------
        component_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.get_component(
            component_id="component_id",
        )
        """
        _response = self._raw_client.get_component(component_id, request_options=request_options)
        return _response.data

    def delete_component(
        self, component_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete a component from the user's library.

        Parameters
        ----------
        component_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.delete_component(
            component_id="component_id",
        )
        """
        _response = self._raw_client.delete_component(component_id, request_options=request_options)
        return _response.data

    def update_map_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: MapConfig,
        id: typing.Optional[str] = OMIT,
        layer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapComponent:
        """
        Update a map component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : MapConfig

        id : typing.Optional[str]

        layer_ids : typing.Optional[typing.Sequence[str]]
            List of Layer IDs contained in this map.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MapComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import MapConfig, RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.update_map_component(
            component_id="component_id",
            user_id="userId",
            name="name",
            config=MapConfig(),
        )
        """
        _response = self._raw_client.update_map_component(
            component_id,
            user_id=user_id,
            name=name,
            config=config,
            id=id,
            layer_ids=layer_ids,
            request_options=request_options,
        )
        return _response.data

    def update_timeseries_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: TimeSeriesConfig,
        id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesComponent:
        """
        Update a timeseries component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : TimeSeriesConfig

        id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi, TimeSeriesConfig

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.update_timeseries_component(
            component_id="component_id",
            user_id="userId",
            name="name",
            config=TimeSeriesConfig(
                title="title",
                datasets=["datasets"],
            ),
        )
        """
        _response = self._raw_client.update_timeseries_component(
            component_id, user_id=user_id, name=name, config=config, id=id, request_options=request_options
        )
        return _response.data

    def update_weather_map_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: WeatherMapConfig,
        id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WeatherMapComponent:
        """
        Update a weather-map component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : WeatherMapConfig

        id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WeatherMapComponent
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi, WeatherMapConfig

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.components.update_weather_map_component(
            component_id="component_id",
            user_id="userId",
            name="name",
            config=WeatherMapConfig(),
        )
        """
        _response = self._raw_client.update_weather_map_component(
            component_id, user_id=user_id, name=name, config=config, id=id, request_options=request_options
        )
        return _response.data


class AsyncComponentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawComponentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawComponentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawComponentsClient
        """
        return self._raw_client

    async def create_map(
        self,
        *,
        name: str,
        config: MapConfig,
        layer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapComponent:
        """
        Create a new map component.

        Parameters
        ----------
        name : str

        config : MapConfig

        layer_ids : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MapComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi, MapConfig

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.create_map(
                name="name",
                config=MapConfig(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_map(
            name=name, config=config, layer_ids=layer_ids, request_options=request_options
        )
        return _response.data

    async def create_timeseries(
        self,
        *,
        name: str,
        title: str,
        datasets: typing.Sequence[str],
        y_axis_label: typing.Optional[str] = OMIT,
        unit: typing.Optional[str] = OMIT,
        formula: typing.Optional[str] = OMIT,
        echarts_options: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesComponent:
        """
        Create a new time series component.

        Parameters
        ----------
        name : str

        title : str
            The user-defined title of the chart.

        datasets : typing.Sequence[str]
            List of timeseries dataset IDs to display on this chart.

        y_axis_label : typing.Optional[str]
            Label for the Y-axis.

        unit : typing.Optional[str]
            The unit for the Y-axis values, e.g., 'MW'.

        formula : typing.Optional[str]
            Optional formula to combine datasets, e.g., 'dataset_1 + dataset_2'

        echarts_options : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional ECharts options for the chart.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.create_timeseries(
                name="name",
                title="title",
                datasets=["datasets"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_timeseries(
            name=name,
            title=title,
            datasets=datasets,
            y_axis_label=y_axis_label,
            unit=unit,
            formula=formula,
            echarts_options=echarts_options,
            request_options=request_options,
        )
        return _response.data

    async def create_weather_map(
        self, *, name: str, config: WeatherMapConfig, request_options: typing.Optional[RequestOptions] = None
    ) -> WeatherMapComponent:
        """
        Create a new weather-map component.

        Parameters
        ----------
        name : str

        config : WeatherMapConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WeatherMapComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi, WeatherMapConfig

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.create_weather_map(
                name="name",
                config=WeatherMapConfig(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_weather_map(name=name, config=config, request_options=request_options)
        return _response.data

    async def list_components(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ListComponentsApiV1DashboardComponentsGetResponseItem]:
        """
        List all components in the user's library.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ListComponentsApiV1DashboardComponentsGetResponseItem]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.list_components()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_components(request_options=request_options)
        return _response.data

    async def get_component(
        self, component_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Retrieve a single component.

        Parameters
        ----------
        component_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.get_component(
                component_id="component_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_component(component_id, request_options=request_options)
        return _response.data

    async def delete_component(
        self, component_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete a component from the user's library.

        Parameters
        ----------
        component_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.delete_component(
                component_id="component_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_component(component_id, request_options=request_options)
        return _response.data

    async def update_map_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: MapConfig,
        id: typing.Optional[str] = OMIT,
        layer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapComponent:
        """
        Update a map component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : MapConfig

        id : typing.Optional[str]

        layer_ids : typing.Optional[typing.Sequence[str]]
            List of Layer IDs contained in this map.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MapComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi, MapConfig

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.update_map_component(
                component_id="component_id",
                user_id="userId",
                name="name",
                config=MapConfig(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_map_component(
            component_id,
            user_id=user_id,
            name=name,
            config=config,
            id=id,
            layer_ids=layer_ids,
            request_options=request_options,
        )
        return _response.data

    async def update_timeseries_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: TimeSeriesConfig,
        id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesComponent:
        """
        Update a timeseries component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : TimeSeriesConfig

        id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi, TimeSeriesConfig

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.update_timeseries_component(
                component_id="component_id",
                user_id="userId",
                name="name",
                config=TimeSeriesConfig(
                    title="title",
                    datasets=["datasets"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_timeseries_component(
            component_id, user_id=user_id, name=name, config=config, id=id, request_options=request_options
        )
        return _response.data

    async def update_weather_map_component(
        self,
        component_id: str,
        *,
        user_id: str,
        name: str,
        config: WeatherMapConfig,
        id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WeatherMapComponent:
        """
        Update a weather-map component.

        Parameters
        ----------
        component_id : str

        user_id : str

        name : str

        config : WeatherMapConfig

        id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WeatherMapComponent
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi, WeatherMapConfig

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.components.update_weather_map_component(
                component_id="component_id",
                user_id="userId",
                name="name",
                config=WeatherMapConfig(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_weather_map_component(
            component_id, user_id=user_id, name=name, config=config, id=id, request_options=request_options
        )
        return _response.data
